==> gitlab-tools-v0.2.0.tar.gz: gitlab-tools/bin/glt-project.sh <==
#!/bin/bash
#
# gitlab-tools:bin/glt-project.sh
#
# Documentation:
#   * https://docs.gitlab.com/ee/api/projects.html
#   * https://docs.gitlab.com/ee/api/projects.html#edit-project
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1090
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit 1
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/utils/common-utils.sh" || exit 1
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/glt-api/project.sh" || exit 1

function usage {
  cat <<EOF
Manage GitLab projects

Usage:
  ${BASH_SOURCE[0]} [options] action

Action:

  -h, --help            Display help message and exit
  group ID              List all projects within given group id.
  id ID                 Display project with given project id.
  list                  List all projects
  name NAME             Display project with given project name

  modify ID KEY VALUE [KEY VALUE]...
                        Modify project configuration. ID (an integer) is
                        the project ID. KEY and VALUE must be pair of key
                        and value to modify

                        Sample:
                            ${BASH_SOURCE[0]} modify 3 description 'DNS configuration' remove_source_branch_after_merge true

  modify-json ID JSONFILE
                        Modify project configuration. ID (an integer) is
                        the project ID. JSONFILE must contain (key,value) pair
                        to modify

  put ID DATA           Modify project configuration. ID (an integer) is
                        the project ID. DATA must be an URL encoded string.

                        Sample:
                            ${BASH_SOURCE[0]} put 3 'description=DNS%20configuration&remove_source_branch_after_merge=true'

  search STR            Search projects having STR in text fields (name, path, tag_list
                        and description).

  search-by-name STR    Search projects having STR in name and path fields only (name, path)

Options:

  -g GITLAB, --gitlab GITLAB
                        Which configuration section should be used. If not
                        defined, the default selection will be used. (python-gitlab)

EOF
}

function gitlab_search_projects {
  local -r gitlab="$1"
  local -r search_name="$2"
  local    first='true'
  local    json=
  local    search_str=

  echo '['
  while read -r json ; do
    search_str="$( jq -r '. | .name, .path, .tag_list[], .description' <<<"${json}" )" || return 1

    if grep --quiet -i "${search_name}" <<<"${search_str}"; then
      if [  "${first}" = 'true' ] ; then
        first='false'
      else
        echo ','
      fi
      echo "${json}"
    fi
  done < <( gitlab_get_all_projects "${gitlab}" | jq -c '.[]' )
  echo ']'
}

function gitlab_search_projects_by_name {
  local -r gitlab="$1"
  local -r search_name="$2"
  local    first='true'
  local    json=
  local    search_str=

  echo '['
  while read -r json ; do
    search_str="$( jq -r '. | .name, .path' <<<"${json}" )" || return 1

    if grep --quiet -i "${search_name}" <<<"${search_str}"; then
      if [  "${first}" = 'true' ] ; then
        first='false'
      else
        echo ','
      fi
      echo "${json}"
    fi
  done < <( gitlab_get_all_groups "${gitlab}" | jq -c '.[]' )
  echo ']'
}

function gitlab_put_project {
  local -r  gitlab="$1"
  local -ri id="$2"
  local -r  api_params="$3"

  # shellcheck disable=SC1090
  source "${PROJECT_HOME}/bin/gitlab-bash-api/init.sh" || exit 1
  # shellcheck disable=SC1090
  source "${GITLAB_BASH_API_PATH}/put.sh" || exit 1

  local -r api_url="/projects/${id}/"

  gitlab_put "${api_url}" "${api_params}"
}

function main {
  local action=
  local gitlab=
  local file=
  local param=
  local param_put_html_encoded=
  local value=

  while [ $# -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage
        exit 0
        ;;

      'group'           ) action='group' ; value="$( P "$@" )" && shift ;;
      'id'              ) action='id' ; value="$( P "$@" )" && shift ;;
      'list'            ) action='list' ;;
      'name'            ) action='name' ; value="$( P "$@" )" && shift ;;
      'search'          ) action='search' ; value="$( P "$@" )" && shift ;;
      'search-by-name'  ) action='search-by-name' ; value="$( P "$@" )" && shift ;;

      'modify')
        action='put'
        value="$( P "$@" )" && shift

        param_put_html_encoded="$( build_html_encoded_from_parameters "$@" )" || exit 1
        shift $#
        ;;

      'modify-json')
        action='put'
        value="$( P "$@" )" && shift
        file="$( P "$@" )" && shift

        param_put_html_encoded="$( build_html_encoded_from_json "${file}" )" || exit 1
        ;;

      'put')
        action='put'
        value="$( P "$@" )" && shift
        param_put_html_encoded="$( P "$@" )" && shift
        ;;

      '--gitlab') gitlab="$( P "$@" )" && shift ;;

      *)
        echofatal "Unknown option '${param}'"
        usage >&2
        exit 1
        ;;
    esac
  done

  case "${action}" in
    'group')
      if ! is_positive_integer "${value}" ; then
        echofatal 'group action imply group id (Integer)'
        usage >&2
        exit 1
      else
        gitlab_get_projects_by_groupid "${gitlab}" "${value}" | jq .
      fi
      ;;

    'id')
      if ! is_positive_integer "${value}" ; then
        echofatal 'id action imply project id (Integer)'
        usage >&2
        exit 1
      else
        gitlab_get_project_by_projectid "${gitlab}" "${value}" | jq .
      fi
      ;;

    'list')
      gitlab_get_all_projects "${gitlab}" | jq .
      ;;

    'name')
      if [ -z "${value}" ] ; then
        echofatal 'name action imply project name'
        usage >&2
        exit 1
      else
        gitlab_get_project_by_projectname "${gitlab}" "${value}" | jq .
      fi
      ;;

    'search')
      if [ -z "${value}" ] ; then
        echofatal 'search action imply search string'
        usage >&2
        exit 1
      else
        gitlab_search_projects "${gitlab}" "${value}" | jq .
      fi
      ;;

    'search-by-name')
      if [ -z "${value}" ] ; then
        echofatal 'search-by-name action imply search string'
        usage >&2
        exit 1
      else
        gitlab_search_projects_by_name "${gitlab}" "${value}" | jq .
      fi
      ;;

    'put')
      if ! is_positive_integer "${value}" ; then
        echofatal 'put action imply project id (Integer)'
        usage >&2
        exit 1
      elif [ -z "${param_put_html_encoded}" ] ; then
        echofatal 'put action imply data to modify'
        usage >&2
        exit 1
      else
        gitlab_put_project "${gitlab}" "${value}" "${param_put_html_encoded}" | jq .
      fi
      ;;

    *)
      echofatal "Missing action or wrong value: '${action}'"
      usage >&2
      exit 1
      ;;
  esac
}

main "$@" || exit 1

==> gitlab-tools-v0.2.1.tar.gz: gitlab-tools/bin/glt-project.sh <==
#!/bin/bash
#
# gitlab-tools:bin/glt-project.sh
#
# Documentation:
#   * https://docs.gitlab.com/ee/api/projects.html
#   * https://docs.gitlab.com/ee/api/projects.html#edit-project
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1090
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit 1
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/utils/common-utils.sh" || exit 1
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/glt-api/project.sh" || exit 1
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/glt-api/group.sh" || exit 1

function usage {
  cat <<EOF
Manage GitLab projects

Usage:
  ${BASH_SOURCE[0]} [options] action

Action:

  -h, --help            Display help message and exit
  group ID              List all projects within given group id.
  id ID                 Display project with given project id.
  list                  List all projects
  name NAME             Display project with given project name

  modify ID KEY VALUE [KEY VALUE]...
                        Modify project configuration. ID (an integer) is
                        the project ID. KEY and VALUE must be pair of key
                        and value to modify

                        Sample:
                            ${BASH_SOURCE[0]} modify 3 description 'DNS configuration' remove_source_branch_after_merge true

  modify-json ID JSONFILE
                        Modify project configuration. ID (an integer) is
                        the project ID. JSONFILE must contain (key,value) pair
                        to modify

  put ID DATA           Modify project configuration. ID (an integer) is
                        the project ID. DATA must be an URL encoded string.

                        Sample:
                            ${BASH_SOURCE[0]} put 3 'description=DNS%20configuration&remove_source_branch_after_merge=true'

  search STR            Search projects having STR in text fields (name, path, tag_list
                        and description).

  search-by-name STR    Search projects having STR in name and path fields only (name, path)

Options:

  -g GITLAB, --gitlab GITLAB
                        Which configuration section should be used. If not
                        defined, the default selection will be used. (python-gitlab)

EOF
}

function gitlab_search_projects {
  local -r gitlab="$1"
  local -r search_name="$2"
  local    first='true'
  local    json=
  local    search_str=

  echo '['
  while read -r json ; do
    search_str="$( jq -r '. | .name, .path, .tag_list[], .description' <<<"${json}" )" || return 1

    if grep --quiet -i "${search_name}" <<<"${search_str}"; then
      if [  "${first}" = 'true' ] ; then
        first='false'
      else
        echo ','
      fi
      echo "${json}"
    fi
  done < <( gitlab_get_all_projects "${gitlab}" | jq -c '.[]' )
  echo ']'
}

function gitlab_search_projects_by_name {
  local -r gitlab="$1"
  local -r search_name="$2"
  local    first='true'
  local    json=
  local    search_str=

  echo '['
  while read -r json ; do
    search_str="$( jq -r '. | .name, .path' <<<"${json}" )" || return 1

    if grep --quiet -i "${search_name}" <<<"${search_str}"; then
      if [  "${first}" = 'true' ] ; then
        first='false'
      else
        echo ','
      fi
      echo "${json}"
    fi
  done < <( gitlab_get_all_groups "${gitlab}" | jq -c '.[]' )
  echo ']'
}

function gitlab_put_project {
  local -r  gitlab="$1"
  local -ri id="$2"
  local -r  api_params="$3"

  # shellcheck disable=SC1090
  source "${PROJECT_HOME}/bin/gitlab-bash-api/init.sh" || exit 1
  # shellcheck disable=SC1090
  source "${GITLAB_BASH_API_PATH}/put.sh" || exit 1

  local -r api_url="/projects/${id}/"

  gitlab_put "${api_url}" "${api_params}"
}

function main {
  local action=
  local file=
  local gitlab=
  local param=
  local param_put_html_encoded=
  local value=

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage
        exit 0
        ;;

      'group'           ) action='group' ; value="$( P "$@" )" && shift ;;
      'id'              ) action='id' ; value="$( P "$@" )" && shift ;;
      'list'            ) action='list' ;;
      'name'            ) action='name' ; value="$( P "$@" )" && shift ;;
      'search'          ) action='search' ; value="$( P "$@" )" && shift ;;
      'search-by-name'  ) action='search-by-name' ; value="$( P "$@" )" && shift ;;

      'modify')
        action='put'
        value="$( P "$@" )" && shift

        param_put_html_encoded="$( build_html_encoded_from_parameters "$@" )" || exit 1
        shift $#
        ;;

      'modify-json')
        action='put'
        value="$( P "$@" )" && shift
        file="$( P "$@" )" && shift

        param_put_html_encoded="$( build_html_encoded_from_json "${file}" )" || exit 1
        ;;

      'put')
        action='put'
        value="$( P "$@" )" && shift
        param_put_html_encoded="$( P "$@" )" && shift
        ;;

      '--gitlab') gitlab="$( P "$@" )" && shift ;;

      *)
        echofatal "Unknown option: '${param}'"
        usage >&2
        exit 1
        ;;
    esac
  done

  case "${action}" in
    'group')
      if ! is_positive_integer "${value}" ; then
        echofatal 'group action imply group id (Integer)'
        usage >&2
        exit 1
      else
        gitlab_get_projects_by_groupid "${gitlab}" "${value}" | jq .
      fi
      ;;

    'id')
      if ! is_positive_integer "${value}" ; then
        echofatal 'id action imply project id (Integer)'
        usage >&2
        exit 1
      else
        gitlab_get_project_by_projectid "${gitlab}" "${value}" | jq .
      fi
      ;;

    'list')
      gitlab_get_all_projects "${gitlab}" | jq .
      ;;

    'name')
      if [ -z "${value}" ] ; then
        echofatal 'name action imply project name'
        usage >&2
        exit 1
      else
        gitlab_get_project_by_projectname "${gitlab}" "${value}" | jq .
      fi
      ;;

    'search')
      if [ -z "${value}" ] ; then
        echofatal 'search action imply search string'
        usage >&2
        exit 1
      else
        gitlab_search_projects "${gitlab}" "${value}" | jq .
      fi
      ;;

    'search-by-name')
      if [ -z "${value}" ] ; then
        echofatal 'search-by-name action imply search string'
        usage >&2
        exit 1
      else
        gitlab_search_projects_by_name "${gitlab}" "${value}" | jq .
      fi
      ;;

    'put')
      if ! is_positive_integer "${value}" ; then
        echofatal 'put action imply project id (Integer)'
        usage >&2
        exit 1
      elif [ -z "${param_put_html_encoded}" ] ; then
        echofatal 'put action imply data to modify'
        usage >&2
        exit 1
      else
        gitlab_put_project "${gitlab}" "${value}" "${param_put_html_encoded}" | jq .
      fi
      ;;

    *)
      echofatal "Missing action or wrong value: '${action}'"
      usage >&2
      exit 1
      ;;
  esac
}

main "$@" || exit 1
